@@ -17,7 +17,7 @@
  *
  *
  *
- *  $Id: AlsaSubscriber.h,v 1.1 2002/03/25 15:42:29 adnans Exp $
+ *  $Id: AlsaSubscriber.h,v 1.2 2002/04/20 11:59:19 adnans Exp $
 */ 
 
 #ifndef __AlsaSubscriber_h__
@@ -36,9 +36,9 @@ class AlsaSubscriber
  public:
 	AlsaSubscriber();
 	~AlsaSubscriber();
-	virtual void Subscribe(AlsaNode *node, int pos=POS_BEGIN);
+	void Subscribe(AlsaNode *node, int pos=POS_BEGIN);
 	void Unsubscribe();
-	virtual void EnterStream(streamer_type str, void *arg);
+	void EnterStream(streamer_type str, void *arg);
 	void ExitStream();	
 };
 
