@@ -17,7 +17,7 @@
  *
  * 
  *
- *  $Id: CorePlayer.h,v 1.24 2003/04/08 15:14:56 adnans Exp $
+ *  $Id: CorePlayer.h,v 1.27 2003/11/02 00:22:43 fbaumgart Exp $
  *  
 */ 
 
@@ -35,17 +35,17 @@
 // Tunable parameters
 
 #define BUF_SIZE (10240)	// Size of a single ringbuffer partition
-#define NR_BUF 20		// Number of partitions in ringbuffer
-#define NR_CBUF 10		// Number of partitions to read ahead
+#define NR_BUF 16		// Number of partitions in ringbuffer
+#define NR_CBUF 8		// Number of partitions to read ahead
 				// (equals 2.6 seconds). NOTE: NR_CBUF
 				// should NEVER exceed (NR_BUF/2), it doesn't
 				// make sense otherwise
 
 #define MAX_INPUT_PLUGINS 16
 
-typedef void(*volume_changed_type)(void *, int new_vol);
+typedef void(*volume_changed_type)(void *, float new_vol);
 typedef void(*speed_changed_type)(void *, float new_speed);
-typedef void(*pan_changed_type)(void *, int new_pan);
+typedef void(*pan_changed_type)(void *, float new_pan);
 typedef void(*position_notify_type)(void *, int pos);
 typedef void(*stop_notify_type)(void *);
 typedef void(*start_notify_type)(void *);		
@@ -87,8 +87,8 @@ class CorePlayer // Much more abstraction to come, well maybe not
 	float pitch_multi;
 	bool jumped;
 	bool producing;
-	int volume;
-	int pan;
+	float volume;
+	float pan;
 	int output_rate;
 	int input_rate;
 	AlsaNode *node;
@@ -106,7 +106,10 @@ class CorePlayer // Much more abstraction to come, well maybe not
 	pthread_mutex_t counter_mutex;
 	pthread_mutex_t thread_mutex;
 	pthread_mutex_t notifier_mutex;
+#if !defined(EMBEDDED)
+	// this buffer is used to apply volume/pan and mixing channels
 	char *input_buffer;
+#endif
 	sample_buf *buffer;
 	sample_buf *read_buf, *write_buf, *new_write_buf;
 	int FrameSeek(int);
@@ -148,14 +151,14 @@ class CorePlayer // Much more abstraction to come, well maybe not
 	void PositionUpdate();	// Notify the interfaces about the position
 	int SetSpeed(float val);	// Set the playback speed: 1.0 = 100%
 	float GetSpeed();	// Get speed
-	int GetVolume() { return volume; }	// Get Volume level
-	void SetVolume(int vol);	// Set volume level
-	int GetPan() { return pan; }	// Get Pan level
+	float GetVolume() { return volume; }	// Get Volume level
+	void SetVolume(float vol);	// Set volume level
+	float GetPan() { return pan; }	// Get Pan level
 	
-	void SetPan(int p);	// Set Pan level: 
-					// 0	= center
-					// -100	= right channel muted
-					// 100  = left channel muted
+	void SetPan(float p);	// Set Pan level: 
+					// 0.0	= center
+					// -1.0 = right channel muted
+					// 1.0  = left channel muted
 	
 	int GetCurrentTime(int frame=-1);
 					// Returns the time position of frame in
