@@ -17,7 +17,7 @@
  *
  *
  *
- *  $Id: Effects.h,v 1.2 2002/06/05 17:56:09 adnans Exp $
+ *  $Id: Effects.h,v 1.3 2003/09/05 09:47:47 adnans Exp $
  *
 */ 
 
@@ -33,7 +33,7 @@ void clear_buffer(void);
 
 void init_effects(void);
 void echo_effect32(void *buf, int size, int delay, int vol);
-void volume_effect32(void *buf, int size, int left, int right=-1);
+void volume_effect32(void *buf, int size, float left, float right=-100.0);
 void buffer_effect(void *buf, int size);
 char *delay_feed(int delay, int max_size);
 
