@@ -17,25 +17,29 @@
  *
  *
  *
- *  $Id: utilities.h,v 1.1 2002/03/25 15:42:29 adnans Exp $
+ *  $Id: utilities.h,v 1.6 2003/02/16 15:06:09 adnans Exp $
  * 
  */ 
 
 #ifndef __utilities_h__
 #define __utilities_h__
 #ifdef __cplusplus
-#include "cpp_utilities.h"
 extern "C" {
 #endif
 
 // Magic macros for stringizing
 #define STRINGISE(a) _STRINGISE(a)	// Not sure why this is needed, but
-									// sometimes fails without it.
-#define _STRINGISE(a) #a			// Now do the stringising
+					// sometimes fails without it.
+#define _STRINGISE(a) #a		// Now do the stringising
 
 // Sleep for specified number of micro-seconds
 // Used by scopes and things, so use C-style linkage
 void dosleep(unsigned int);
+void parse_file_uri_free(char *);
+char *get_homedir(void);
+char *get_prefsdir(void);
+char *parse_file_uri(const char *);
+int is_playlist(const char *);
 
 #ifdef __cplusplus
 }
