@@ -16,14 +16,14 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  *
- * $Id: AlsaSubscriber.cpp,v 1.4 2002/03/05 11:47:54 adnans Exp $
+ * $Id: AlsaSubscriber.cpp,v 1.6 2003/02/16 15:06:10 adnans Exp $
  *
 */ 
 
 #include "AlsaSubscriber.h"
-#include <stdio.h>
-#include <string.h>
-#include <assert.h>
+#include <cstdio>
+#include <cstring>
+#include <cassert>
 #include <unistd.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -81,7 +81,7 @@ void AlsaSubscriber::ExitStream()
 {
 	if (the_node && the_node->IsInStream(the_ID)) {
 		if (!the_node->RemoveStreamer(the_ID)) {
-			printf("ERROR! Failed to remove streamer\n");
+			puts("ERROR! Failed to remove streamer");
 		} else {
 			the_ID = -1;
 		}	
