@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  *
- *  $Id: CorePlayer.cpp,v 1.65 2003/04/18 23:44:28 adnans Exp $
+ *  $Id: CorePlayer.cpp,v 1.73 2003/11/10 19:57:45 adnans Exp $
  *
 */ 
 
@@ -224,8 +224,8 @@ CorePlayer::CorePlayer(AlsaNode *the_node)
 	input_rate = output_rate = 44100;
 	SetSpeedMulti(1.0);
 	SetSpeed(1.0);
-	SetVolume(100);
-	SetPan(0);
+	SetVolume(1.0);
+	SetPan(0.0);
 	buffer = NULL;
 	the_object = NULL;
 
@@ -270,10 +270,12 @@ CorePlayer::CorePlayer(AlsaNode *the_node)
 		plugins_loaded = 1;
 		load_input_addons();
 	}
+#if !defined(EMBEDDED)
 	if ((input_buffer = new char[128 * 1024]) == NULL) {
 		alsaplayer_error("cannot allocate mix buffer");
 		exit(1);
 	}	
+#endif
 	pthread_mutex_unlock(&plugins_mutex);
 }
 
@@ -302,7 +304,9 @@ CorePlayer::~CorePlayer()
 	pthread_mutex_destroy(&the_object->object_mutex);
 	delete []buffer;
 	delete the_object;
+#if !defined(EMBEDDED)
 	delete []input_buffer;
+#endif
 }
 
 
@@ -604,6 +608,7 @@ bool CorePlayer::Start()
 	}
 	pthread_create(&producer_thread, NULL,
 		(void * (*)(void *))producer_func, this);
+
 	// allow producer to actually start producing
 	dosleep(20000);
 	//alsaplayer_error("Prebuffering...");
@@ -681,10 +686,14 @@ void CorePlayer::Stop()
 }
 
 
-void CorePlayer::SetVolume(int val)
+void CorePlayer::SetVolume(float val)
 {
+	if (val > 2.0) {
+		alsaplayer_error("ERROR: volume out of bound (%.2f)", val);
+		return;
+	}	
 	volume = val;
-
+	
 	std::set<coreplayer_notifier *>::const_iterator i;
 	LockNotifiers();
 	for (i = notifiers.begin(); i != notifiers.end(); i++) {
@@ -694,7 +703,7 @@ void CorePlayer::SetVolume(int val)
 	UnlockNotifiers();
 }
 
-void CorePlayer::SetPan(int val)
+void CorePlayer::SetPan(float val)
 {
 	pan = val;
 
@@ -833,6 +842,8 @@ bool CorePlayer::Open(const char *path)
 	}
 	Lock();	
 	
+	the_object->local_data = NULL;
+
 	if (best_plugin->open(the_object, path)) {
 		if ((frame_size = best_plugin->frame_size(the_object)) > BUF_SIZE) {
 			alsaplayer_error("CRITICAL ERROR: this plugin advertised a buffer size\n"
@@ -958,8 +969,12 @@ void CorePlayer::update_pitch()
 }	
 
 
-// FIXME: some documentation needed
-//
+/*
+	FIXME: some documentation needed
+
+	returns the number of samples put into output buffer
+	negative return values indicate errors
+*/
 int CorePlayer::Read32(void *data, int samples)
 {
 	if (repitched) {
@@ -1000,13 +1015,47 @@ int CorePlayer::Read32(void *data, int samples)
 		write_buf_changed = 1;
 		pthread_mutex_unlock(&counter_mutex);
 	}
-	
+#ifdef EMBEDDED
+	// provide a fast path implementation for the common case (no pitch)
+	if ((use_read_direction == DIR_FORWARD) && (use_pitch == 1.0)) {
+
+		// calculate amount of samples available in current read buffer
+		int samplesInBuffer = read_buf->buf->write_index - check_index;
+
+		if (samplesInBuffer >= samples)
+		{
+			// copy the whole block (usually 4KB) at once
+			memcpy(out_buf, in_buf, samples * 4);
+			tmp_index = samples - 1;
+		}
+		else
+		{
+			if (samplesInBuffer <= 0) {
+				memset(data, 0, samples * 4);
+				return samples;
+			}	
+			// use rest of current block
+			if (samplesInBuffer)
+				memcpy(out_buf, in_buf, samplesInBuffer * 4);
+
+			// get next block
+			read_buf = read_buf->next;
+			pthread_mutex_unlock(&counter_mutex);
+			read_buf->buf->SetReadDirection(DIR_FORWARD);
+			read_buf->buf->ResetRead();
+			in_buf = (int *)read_buf->buf->buffer_data;
+			check_index = samples - samplesInBuffer;
+			memcpy(out_buf + samplesInBuffer, in_buf, (samples - samplesInBuffer) * 4);
+			tmp_index = -1;
+		}
+	}
+	else 
+#endif		
 	if (use_read_direction == DIR_FORWARD) {
 		while (buf_index < samples) {
 			tmp_index = (int) ((float)use_pitch*(float)(buf_index-base_corr));
 			if ((check_index + tmp_index) > (read_buf->buf->write_index)-1) {
-				if (read_buf->next->start < 0 ||
-					read_buf->next == write_buf) {
+				if (read_buf->next->start < 0 || read_buf->next == write_buf) {
 					if (read_buf->next->start == -2) {
 						//alsaplayer_error("Next in queue (2)");
 						return -2;
@@ -1030,7 +1079,8 @@ int CorePlayer::Read32(void *data, int samples)
 				in_buf = (int *)read_buf->buf->buffer_data;
 				base_corr = buf_index;
 				check_index = 0;
-				tmp_index = (int)((float)use_pitch*(float)(buf_index-base_corr)); // Recalc
+				// Recalc
+				tmp_index = (int)((float)use_pitch*(float)(buf_index-base_corr));
 			}
 			out_buf[buf_index++] =  *(in_buf + tmp_index);
 		}
@@ -1058,14 +1108,17 @@ int CorePlayer::Read32(void *data, int samples)
 				in_buf += (buf_size + (check_index - tmp_index));
 				base_corr = buf_index;
 				check_index = buf_size-1;
-				tmp_index = (int)((float)use_pitch*(float)(buf_index-base_corr)); // Recalc
+				// Recalc
+				tmp_index = (int)((float)use_pitch*(float)(buf_index-base_corr));
 			}
 			out_buf[buf_index++] = *(in_buf - tmp_index);
 		}
 	}
+
 	read_buf->buf->Seek(check_index + ((use_read_direction == DIR_FORWARD) ?
 		 tmp_index+1 : -((tmp_index+1) > check_index ? check_index : tmp_index+1)));
 	if (!samples) alsaplayer_error("Humm, I copied nothing?");
+
 	return samples;
 }
 
@@ -1180,30 +1233,44 @@ extern int global_reverb_on;
 extern int global_reverb_delay;
 extern int global_reverb_feedback;
 
+#ifdef EMBEDDED
+// the simple streamer only handles a single stream with no software
+// volume/pan calculation; intended for slow systems/embedded system use
+bool CorePlayer::streamer_func(void *arg, void *data, int size)
+{
+	CorePlayer *obj = (CorePlayer *)arg;
+
+	if (obj->Read32(data, size / sizeof(short)) >= 0)
+		return true;
+
+	obj->streaming = false;
+	return false;
+}
+
+#else
+
 bool CorePlayer::streamer_func(void *arg, void *data, int size)
 {
 	CorePlayer *obj = (CorePlayer *)arg;
 	int count;
 
 	if ((count = obj->Read32(obj->input_buffer, size / sizeof(short))) >= 0) {
-		int v, p, left, right;
+		float v, p, left, right;
 		p = obj->pan;
 		v = obj->volume;
-		if (v != 100 || p != 0) {
-			if (p == 0) {
-				left = right = 100;
+		if (v != 1.0 || p != 0.0) {
+			if (p == 0.0) {
+				left = right = 1.0;
 			} else 	if (p > 0) {
-				right = 100;
-				left = 100 - p;
+				right = 1.0;
+				left = 1.0 - p;
 			} else {
-				left = 100;
-				right = 100 + p;
+				left = 1.0;
+				right = 1.0 + p;
 			}
-			if (v != 100) {
+			if (v != 1.0) {
 				left *= v;
-				left /= 100;
 				right *= v;
-				right /= 100;
 			}
 			//printf("v = %d, p = %d, left = %d, right = %d\n",
 			//	p, v, left, right);
@@ -1228,6 +1295,7 @@ bool CorePlayer::streamer_func(void *arg, void *data, int size)
 		return false;
 	}	
 }
+#endif
 
 
 bool CorePlayer::IsPaused()
