@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  *
- *  $Id: Main.cpp,v 1.105 2003/04/23 23:13:53 adnans Exp $
+ *  $Id: Main.cpp,v 1.109 2003/09/18 17:31:23 sjah Exp $
  *
  *  VIM: set ts=8
  *
@@ -275,7 +275,7 @@ static void help()
 		" ]\n"
 		"  -I,--script file        script to pass to interface plugin\n"
 		"  -n,--session n          use this session id [default=0]\n"
-		"  -l,--startvolume vol    start with this volume [default=100]\n"
+		"  -l,--startvolume vol    start with this volume [default=1.0]\n"
 		"  -p,--path path          set the path alsaplayer looks for add-ons\n"
 		"  -q,--quiet              quiet operation. less output\n"
 		"  -s,--session-name name  name this session \"name\"\n"
@@ -288,7 +288,7 @@ static void help()
 		"  -e,--enqueue file(s)  queue files in running alsaplayer\n"
 		"  -E,--replace file(s)  clears and queues files in running alsaplayer\n"
 		"  --status              get some information about session\n"
-		"  --volume vol          set software volume [0-100]\n"
+		"  --volume vol          set software volume [0.0-1.0]\n"
 		"  --start               start playing\n"
 		"  --stop                stop playing\n"
 		"  --pause               pause/unpause playing\n"
@@ -296,6 +296,8 @@ static void help()
 		"  --next                jump to next track\n"
 		"  --seek second         jump to specified second in current track\n"
 		"  --relative second     jump second seconds from current position\n"
+		"  --speed speed         floating point speed parameter\n"
+		"    1.0 = normal speed, -1.0 normal speed backwards\n"
 		"  --jump track          jump to specified playlist track\n"
 		"  --clear               clear whole playlist\n"
 		"  --quit                quit session\n"
@@ -348,7 +350,7 @@ int main(int argc, char **argv)
 	char *prefsdir;
 	char thefile[1024];
 	char str[1024];
-	int start_vol = 100;
+	float start_vol = 1.0;
 	int ap_result = 0;
 	int use_fragsize = -1; // Initialized
 	int use_fragcount = -1; // later
@@ -371,9 +373,11 @@ int main(int argc, char **argv)
 	int do_setvol = 0;
 	int do_quit = 0;
 	int do_status = 0;
-	
+	int do_speed = 0;
+	float speed_val = 0.0;
+		
 	int use_freq = OUTPUT_RATE;
-	int use_vol = 100;
+	float use_vol = 1.0;
 	int use_session = 0;
 	int do_crossfade = 0;
 	int do_save = 1;
@@ -418,6 +422,7 @@ int main(int argc, char **argv)
 		{ "jump", 1, 0, 'J' },
 		{ "seek", 1, 0, 'X' },
 		{ "relative", 1, 0, 'Z' },
+		{ "speed", 1, 0, 'H' },
 		{ "clear", 0, 0, 'C' },
 		{ "startvolume", 1, 0, 'l' },
 		{ "quit", 0, 0, 'A' },
@@ -500,23 +505,29 @@ int main(int argc, char **argv)
 			case 'h':
 				help();
 				return 0;
+			case 'H':
+				if ((sscanf(optarg, "%f", &speed_val))) {
+					do_remote_control = 1;
+					do_speed = 1;
+				}
+				break;
 			case 'i':
 				use_interface = optarg;
 				break;
 			case 'l':
-				start_vol = atoi(optarg);
-				if (start_vol < 0 || start_vol > 100) {
-					alsaplayer_error("volume out of range: using 100");
-					start_vol = 100;
+				start_vol = atof(optarg);
+				if (start_vol < 0.0 || start_vol > 1.0) {
+					alsaplayer_error("volume out of range: using 1.0");
+					start_vol = 1.0;
 				}
 				break;
 			case 'Y':
 				do_remote_control = 1;
 				do_setvol = 1;
-				use_vol = atoi(optarg);
-				if (use_vol < 0 || use_vol > 100) {
-					alsaplayer_error("volume out of range: using 100");
-					use_vol = 100;
+				use_vol = atof(optarg);
+				if (use_vol < 0.0 || use_vol > 1.0) {
+					alsaplayer_error("volume out of range: using 1.0");
+					use_vol = 1.0;
 				}
 				break;
 			case 'n':
@@ -613,7 +624,7 @@ int main(int argc, char **argv)
 	prefsdir = get_prefsdir();
 	
 	mkdir(prefsdir, 0700);	/* XXX We don't do any error checking here */
-	snprintf(thefile, sizeof(prefsdir)-21, "%s/config", prefsdir);
+	snprintf(thefile, sizeof(thefile)-21, "%s/config", prefsdir);
 	if (use_config) 
 		ap_prefs = prefs_load(use_config);
 	else
@@ -647,7 +658,10 @@ int main(int argc, char **argv)
 		}
 		if (use_session == (MAX_REMOTE_SESSIONS+1)) {
 			//alsaplayer_error("No remote session found");
-			do_remote_control = 0;
+			if (do_remote_control) {
+				alsaplayer_error("No active sessions");
+				return 1;
+			}	
 			do_enqueue = 0;
 		} else {
 			//alsaplayer_error("Found session %d", use_session);
@@ -674,8 +688,8 @@ int main(int argc, char **argv)
 				fprintf(stdout, "name: %s\n", res);
 			if (ap_get_playlist_length(use_session, &ires))
 				fprintf(stdout, "playlist_length: %d\n", ires);
-			if (ap_get_volume(use_session, &ires))
-				fprintf(stdout, "volume: %d\n", ires);
+			if (ap_get_volume(use_session, &fres))
+				fprintf(stdout, "volume: %.2f\n", fres);
 			if (ap_get_speed(use_session, &fres))
 				fprintf(stdout, "speed: %d%%\n", (int)(fres * 100));
 			fprintf(stdout, "-------------- Current Track ------------\n");
@@ -730,6 +744,13 @@ int main(int argc, char **argv)
 			if (do_seek != 0)
 				ap_set_position_relative(use_session, do_seek);
 			return 0;
+		} else if (do_speed) {
+			if (speed_val < -10.0 || speed_val > 10.0) {
+				alsaplayer_error("Speed out of range, must be between -10.00 and 10.00");
+				return 1;
+			}
+			ap_set_speed(use_session, speed_val);
+			return 0;
 		} else if (do_seek >= 0) {
 			ap_set_position(use_session, do_seek);
 			return 0;
