@@ -16,7 +16,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  *
- *  $Id: Playlist.cpp,v 1.87 2003/04/23 23:20:32 adnans Exp $
+ *  $Id: Playlist.cpp,v 1.89 2003/11/02 18:30:46 adnans Exp $
  *
 */
 
@@ -422,7 +422,7 @@ Playlist::Playlist(AlsaNode *the_node) {
 
 	UnPause();
 	UnLoopSong();		// Default values
-	LoopPlaylist();		// for looping
+	UnLoopPlaylist();	// for looping
 	UnCrossfade();		// and crossfading
 
 	pthread_mutex_init(&playlist_mutex, NULL);
