@@ -22,13 +22,13 @@
     Version 1.0.0
  *
  *
- *  $Id: reverbst.cpp,v 1.2 2001/09/07 12:44:43 adnans Exp $
+ *  $Id: reverbst.cpp,v 1.3 2003/02/16 15:06:11 adnans Exp $
  *
  */
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <math.h>
+#include <cstdio>
+#include <cstdlib>
+#include <cmath>
 #include <sys/ioctl.h>
 #include <unistd.h>
 #include <fcntl.h>
