@@ -1,5 +1,5 @@
 /*
- * mad - MPEG audio decoder
+ * libmad - MPEG audio decoder library
  * Copyright (C) 2000-2001 Robert Leslie
  *
  * This program is free software; you can redistribute it and/or modify
@@ -20,20 +20,24 @@
  * so by contacting the author: Robert Leslie <rob@mars.org>
  */
 
+# ifdef __cplusplus
+extern "C" {
+# endif
+
 /* # define FPM_INTEL */
 
 # define SIZEOF_INT 4
 # define SIZEOF_LONG 4
 # define SIZEOF_LONG_LONG 8
 
-/* Id: version.h,v 1.16 2001/04/05 04:57:11 rob Exp */
+/* Id: version.h,v 1.20 2001/10/27 22:47:32 rob Exp */
 
 # ifndef LIBMAD_VERSION_H
 # define LIBMAD_VERSION_H
 
 # define MAD_VERSION_MAJOR	0
-# define MAD_VERSION_MINOR	13
-# define MAD_VERSION_PATCH	0
+# define MAD_VERSION_MINOR	14
+# define MAD_VERSION_PATCH	2
 # define MAD_VERSION_EXTRA	" (beta)"
 
 # define MAD_VERSION_STRINGIZE(str)	#str
@@ -55,7 +59,7 @@ extern char const mad_build[];
 
 # endif
 
-/* Id: fixed.h,v 1.23 2001/04/05 04:57:11 rob Exp */
+/* Id: fixed.h,v 1.30 2001/11/02 09:51:06 rob Exp */
 
 # ifndef LIBMAD_FIXED_H
 # define LIBMAD_FIXED_H
@@ -72,6 +76,18 @@ typedef   signed long mad_fixed64hi_t;
 typedef unsigned long mad_fixed64lo_t;
 # endif
 
+# if defined(_MSC_VER)
+#  define mad_fixed64_t  signed __int64
+# elif 1 || defined(__GNUC__)
+#  define mad_fixed64_t  signed long long
+# endif
+
+# if defined(FPM_FLOAT)
+typedef double mad_sample_t;
+# else
+typedef mad_fixed_t mad_sample_t;
+# endif
+
 /*
  * Fixed-point format: 0xABBBBBBB
  * A == whole part      (sign + 3 bits)
@@ -132,20 +148,31 @@ typedef unsigned long mad_fixed64lo_t;
 # define mad_f_add(x, y)	((x) + (y))
 # define mad_f_sub(x, y)	((x) - (y))
 
-# if defined(FPM_64BIT)
+# if defined(FPM_FLOAT)
+#  error "FPM_FLOAT not yet supported"
+
+#  undef MAD_F
+#  define MAD_F(x)		mad_f_todouble(x)
+
+#  define mad_f_mul(x, y)	((x) * (y))
+#  define mad_f_scale64
+
+#  undef ASO_ZEROCHECK
+
+# elif defined(FPM_64BIT)
 
 /*
- * This version should be the most accurate if 64-bit (long long) types are
- * supported by the compiler, although it may not be the most efficient.
+ * This version should be the most accurate if 64-bit types are supported by
+ * the compiler, although it may not be the most efficient.
  */
 #  if defined(OPT_ACCURACY)
 #   define mad_f_mul(x, y)  \
     ((mad_fixed_t)  \
-     ((((signed long long) (x) * (y)) +  \
+     ((((mad_fixed64_t) (x) * (y)) +  \
        (1L << (MAD_F_SCALEBITS - 1))) >> MAD_F_SCALEBITS))
 #  else
 #   define mad_f_mul(x, y)  \
-    ((mad_fixed_t) (((signed long long) (x) * (y)) >> MAD_F_SCALEBITS))
+    ((mad_fixed_t) (((mad_fixed64_t) (x) * (y)) >> MAD_F_SCALEBITS))
 #  endif
 
 #  define MAD_F_SCALEBITS  MAD_F_FRACBITS
@@ -154,21 +181,44 @@ typedef unsigned long mad_fixed64lo_t;
 
 # elif defined(FPM_INTEL)
 
+#  if defined(_MSC_VER)
+#   pragma warning(push)
+#   pragma warning(disable: 4035)  /* no return value */
+static __forceinline
+mad_fixed_t mad_f_mul_inline(mad_fixed_t x, mad_fixed_t y)
+{
+  enum {
+    fracbits = MAD_F_FRACBITS
+  };
+
+  __asm {
+    mov eax, x
+    imul y
+    shrd eax, edx, fracbits
+  }
+
+  /* implicit return of eax */
+}
+#   pragma warning(pop)
+
+#   define mad_f_mul		mad_f_mul_inline
+#   define mad_f_scale64
+#  else
 /*
  * This Intel version is fast and accurate; the disposition of the least
  * significant bit depends on OPT_ACCURACY via mad_f_scale64().
  */
-#  define MAD_F_MLX(hi, lo, x, y)  \
+#   define MAD_F_MLX(hi, lo, x, y)  \
     asm ("imull %3"  \
 	 : "=a" (lo), "=d" (hi)  \
 	 : "%a" (x), "rm" (y)  \
 	 : "cc")
 
-#  if defined(OPT_ACCURACY)
+#   if defined(OPT_ACCURACY)
 /*
  * This gives best accuracy but is not very fast.
  */
-#   define MAD_F_MLA(hi, lo, x, y)  \
+#    define MAD_F_MLA(hi, lo, x, y)  \
     ({ mad_fixed64hi_t __hi;  \
        mad_fixed64lo_t __lo;  \
        MAD_F_MLX(__hi, __lo, (x), (y));  \
@@ -178,13 +228,13 @@ typedef unsigned long mad_fixed64lo_t;
 	    : "r" (__lo), "r" (__hi), "0" (lo), "1" (hi)  \
 	    : "cc");  \
     })
-#  endif  /* OPT_ACCURACY */
+#   endif  /* OPT_ACCURACY */
 
-#  if defined(OPT_ACCURACY)
+#   if defined(OPT_ACCURACY)
 /*
  * Surprisingly, this is faster than SHRD followed by ADC.
  */
-#   define mad_f_scale64(hi, lo)  \
+#    define mad_f_scale64(hi, lo)  \
     ({ mad_fixed64hi_t __hi_;  \
        mad_fixed64lo_t __lo_;  \
        mad_fixed_t __result;  \
@@ -200,8 +250,8 @@ typedef unsigned long mad_fixed64lo_t;
 	    : "cc");  \
        __result;  \
     })
-#  else
-#   define mad_f_scale64(hi, lo)  \
+#   else
+#    define mad_f_scale64(hi, lo)  \
     ({ mad_fixed_t __result;  \
        asm ("shrdl %3,%2,%1"  \
 	    : "=rm" (__result)  \
@@ -209,9 +259,10 @@ typedef unsigned long mad_fixed64lo_t;
 	    : "cc");  \
        __result;  \
     })
-#  endif  /* OPT_ACCURACY */
+#   endif  /* OPT_ACCURACY */
 
-#  define MAD_F_SCALEBITS  MAD_F_FRACBITS
+#   define MAD_F_SCALEBITS  MAD_F_FRACBITS
+#  endif
 
 /* --- ARM ----------------------------------------------------------------- */
 
@@ -255,6 +306,13 @@ typedef unsigned long mad_fixed64lo_t;
 	 : "+r" (lo), "+r" (hi)  \
 	 : "%r" (x), "r" (y))
 
+#  define MAD_F_MLN(hi, lo)  \
+    asm ("rsbs	%0, %2, #0\n\t"  \
+	 "rsc	%1, %3, #0"  \
+	 : "=r" (lo), "=r" (hi)  \
+	 : "0" (lo), "1" (hi)  \
+	 : "cc")
+
 #  define mad_f_scale64(hi, lo)  \
     ({ mad_fixed_t __result;  \
        asm ("movs	%0, %1, lsr %3\n\t"  \
@@ -395,8 +453,12 @@ typedef unsigned long mad_fixed64lo_t;
  *
  * Pre-rounding is required to stay within the limits of compliance.
  */
-#  define mad_f_mul(x, y)	((((x) + (1L << 11)) >> 12) *  \
+#  if defined(OPT_SPEED)
+#   define mad_f_mul(x, y)	(((x) >> 12) * ((y) >> 16))
+#  else
+#   define mad_f_mul(x, y)	((((x) + (1L << 11)) >> 12) *  \
 				 (((y) + (1L << 15)) >> 16))
+#  endif
 
 /* ------------------------------------------------------------------------- */
 
@@ -418,6 +480,7 @@ typedef unsigned long mad_fixed64lo_t;
 # if !defined(MAD_F_MLA)
 #  define MAD_F_ML0(hi, lo, x, y)	((lo)  = mad_f_mul((x), (y)))
 #  define MAD_F_MLA(hi, lo, x, y)	((lo) += mad_f_mul((x), (y)))
+#  define MAD_F_MLN(hi, lo)		((lo)  = -(lo))
 #  define MAD_F_MLZ(hi, lo)		((void) (hi), (mad_fixed_t) (lo))
 # endif
 
@@ -425,6 +488,10 @@ typedef unsigned long mad_fixed64lo_t;
 #  define MAD_F_ML0(hi, lo, x, y)	MAD_F_MLX((hi), (lo), (x), (y))
 # endif
 
+# if !defined(MAD_F_MLN)
+#  define MAD_F_MLN(hi, lo)		((hi) = ((lo) = -(lo)) ? ~(hi) : -(hi))
+# endif
+
 # if !defined(MAD_F_MLZ)
 #  define MAD_F_MLZ(hi, lo)		mad_f_scale64((hi), (lo))
 # endif
@@ -450,7 +517,7 @@ mad_fixed_t mad_f_abs(mad_fixed_t);
 
 # endif
 
-/* Id: bit.h,v 1.7 2001/04/05 04:57:11 rob Exp */
+/* Id: bit.h,v 1.8 2001/10/17 19:14:47 rob Exp */
 
 # ifndef LIBMAD_BIT_H
 # define LIBMAD_BIT_H
@@ -479,7 +546,7 @@ unsigned short mad_bit_crc(struct mad_bitptr, unsigned int, unsigned short);
 
 # endif
 
-/* Id: timer.h,v 1.10 2001/04/05 04:57:11 rob Exp */
+/* Id: timer.h,v 1.12 2001/11/03 03:57:11 rob Exp */
 
 # ifndef LIBMAD_TIMER_H
 # define LIBMAD_TIMER_H
@@ -541,7 +608,7 @@ enum mad_units {
   MAD_UNITS_59_94_FPS	 =   -60
 };
 
-# define mad_timer_reset(timer)	(*(timer) = mad_timer_zero)
+# define mad_timer_reset(timer)	((void) (*(timer) = mad_timer_zero))
 
 int mad_timer_compare(mad_timer_t, mad_timer_t);
 
@@ -561,7 +628,7 @@ void mad_timer_string(mad_timer_t, char *, char const *,
 
 # endif
 
-/* Id: stream.h,v 1.12 2001/04/10 05:18:21 rob Exp */
+/* Id: stream.h,v 1.15 2001/11/08 23:28:03 rob Exp */
 
 # ifndef LIBMAD_STREAM_H
 # define LIBMAD_STREAM_H
@@ -570,6 +637,8 @@ void mad_timer_string(mad_timer_t, char *, char const *,
 # define MAD_BUFFER_MDLEN	(511 + 2048 + MAD_BUFFER_GUARD)
 
 enum mad_error {
+  MAD_ERROR_NONE	   = 0x0000,	/* no error */
+
   MAD_ERROR_BUFLEN	   = 0x0001,	/* input buffer too small (or EOF) */
   MAD_ERROR_BUFPTR	   = 0x0002,	/* invalid (null) buffer pointer */
 
@@ -622,18 +691,19 @@ struct mad_stream {
 
 enum {
   MAD_OPTION_IGNORECRC      = 0x0001,	/* ignore CRC errors */
-  MAD_OPTION_HALFSAMPLERATE = 0x0002,	/* generate PCM at 1/2 sample rate */
+  MAD_OPTION_HALFSAMPLERATE = 0x0002	/* generate PCM at 1/2 sample rate */
 # if 0  /* not yet implemented */
   MAD_OPTION_LEFTCHANNEL    = 0x0010,	/* decode left channel only */
   MAD_OPTION_RIGHTCHANNEL   = 0x0020,	/* decode right channel only */
-  MAD_OPTION_SINGLECHANNEL  = 0x0030,	/* combine channels */
+  MAD_OPTION_SINGLECHANNEL  = 0x0030	/* combine channels */
 # endif
 };
 
 void mad_stream_init(struct mad_stream *);
 void mad_stream_finish(struct mad_stream *);
 
-# define mad_stream_options(stream, opts)  ((stream)->options = (opts))
+# define mad_stream_options(stream, opts)  \
+    ((void) ((stream)->options = (opts)))
 
 void mad_stream_buffer(struct mad_stream *,
 		       unsigned char const *, unsigned long);
@@ -641,9 +711,11 @@ void mad_stream_skip(struct mad_stream *, unsigned long);
 
 int mad_stream_sync(struct mad_stream *);
 
+char const *mad_stream_errorstr(struct mad_stream const *);
+
 # endif
 
-/* Id: frame.h,v 1.13 2001/04/05 04:57:11 rob Exp */
+/* Id: frame.h,v 1.16 2001/10/17 19:13:41 rob Exp */
 
 # ifndef LIBMAD_FRAME_H
 # define LIBMAD_FRAME_H
@@ -667,24 +739,26 @@ enum mad_emphasis {
   MAD_EMPHASIS_CCITT_J_17 = 3		/* CCITT J.17 emphasis */
 };
 
+struct mad_header {
+  enum mad_layer layer;			/* audio layer (1, 2, or 3) */
+  enum mad_mode mode;			/* channel mode (see above) */
+  int mode_extension;			/* additional mode info */
+  enum mad_emphasis emphasis;		/* de-emphasis to use (see above) */
+
+  unsigned long bitrate;		/* stream bitrate (bps) */
+  unsigned int samplerate;		/* sampling frequency (Hz) */
+
+  unsigned short crc_check;		/* frame CRC accumulator */
+  unsigned short crc_target;		/* final target CRC checksum */
+
+  int flags;				/* flags (see below) */
+  int private_bits;			/* private bits (see below) */
+
+  mad_timer_t duration;			/* audio playing time of frame */
+};
+
 struct mad_frame {
-  struct mad_header {
-    enum mad_layer layer;		/* audio layer (1, 2, or 3) */
-    enum mad_mode mode;			/* channel mode (see above) */
-    int mode_extension;			/* additional mode info */
-    enum mad_emphasis emphasis;		/* de-emphasis to use (see above) */
-
-    unsigned long bitrate;		/* stream bitrate (bps) */
-    unsigned int samplerate;		/* sampling frequency (Hz) */
-
-    unsigned short crc_check;		/* frame CRC accumulator */
-    unsigned short crc_target;		/* final target CRC checksum */
-
-    int flags;				/* flags (see below) */
-    int private_bits;			/* private bits (see below) */
-
-    mad_timer_t duration;		/* audio playing time of frame */
-  } header;
+  struct mad_header header;		/* MPEG audio header */
 
   int options;				/* decoding options (from stream) */
 
@@ -699,26 +773,26 @@ struct mad_frame {
      ((header)->flags & MAD_FLAG_LSF_EXT)) ? 18 : 36))
 
 enum {
-  MAD_FLAG_NPRIVATE_III	  = 0x0007,	/* number of Layer III private bits */
-  MAD_FLAG_INCOMPLETE	  = 0x0008,	/* header but not data is decoded */
+  MAD_FLAG_NPRIVATE_III	= 0x0007,	/* number of Layer III private bits */
+  MAD_FLAG_INCOMPLETE	= 0x0008,	/* header but not data is decoded */
 
-  MAD_FLAG_PROTECTION	  = 0x0010,	/* frame has CRC protection */
-  MAD_FLAG_COPYRIGHT	  = 0x0020,	/* frame is copyright */
-  MAD_FLAG_ORIGINAL	  = 0x0040,	/* frame is original (else copy) */
-  MAD_FLAG_PADDING	  = 0x0080,	/* frame has additional slot */
+  MAD_FLAG_PROTECTION	= 0x0010,	/* frame has CRC protection */
+  MAD_FLAG_COPYRIGHT	= 0x0020,	/* frame is copyright */
+  MAD_FLAG_ORIGINAL	= 0x0040,	/* frame is original (else copy) */
+  MAD_FLAG_PADDING	= 0x0080,	/* frame has additional slot */
 
-  MAD_FLAG_I_STEREO	  = 0x0100,	/* uses intensity joint stereo */
-  MAD_FLAG_MS_STEREO	  = 0x0200,	/* uses middle/side joint stereo */
-  MAD_FLAG_FREEFORMAT	  = 0x0400,	/* uses free format bitrate */
+  MAD_FLAG_I_STEREO	= 0x0100,	/* uses intensity joint stereo */
+  MAD_FLAG_MS_STEREO	= 0x0200,	/* uses middle/side joint stereo */
+  MAD_FLAG_FREEFORMAT	= 0x0400,	/* uses free format bitrate */
 
-  MAD_FLAG_LSF_EXT	  = 0x1000,	/* lower sampling freq. extension */
-  MAD_FLAG_MC_EXT	  = 0x2000,	/* multichannel audio extension */
-  MAD_FLAG_MPEG_2_5_EXT	  = 0x4000	/* MPEG 2.5 (unofficial) extension */
+  MAD_FLAG_LSF_EXT	= 0x1000,	/* lower sampling freq. extension */
+  MAD_FLAG_MC_EXT	= 0x2000,	/* multichannel audio extension */
+  MAD_FLAG_MPEG_2_5_EXT	= 0x4000	/* MPEG 2.5 (unofficial) extension */
 };
 
 enum {
-  MAD_PRIVATE_HEADER	  = 0x0100,	/* header private bit */
-  MAD_PRIVATE_III	  = 0x001f	/* Layer III private bits (up to 5) */
+  MAD_PRIVATE_HEADER	= 0x0100,	/* header private bit */
+  MAD_PRIVATE_III	= 0x001f	/* Layer III private bits (up to 5) */
 };
 
 void mad_header_init(struct mad_header *);
@@ -736,23 +810,42 @@ void mad_frame_mute(struct mad_frame *);
 
 # endif
 
-/* Id: synth.h,v 1.8 2001/04/05 04:57:11 rob Exp */
+/* Id: synth.h,v 1.11 2001/11/08 23:28:03 rob Exp */
 
 # ifndef LIBMAD_SYNTH_H
 # define LIBMAD_SYNTH_H
 
+struct mad_pcm {
+  unsigned int samplerate;		/* sampling frequency (Hz) */
+  unsigned short channels;		/* number of channels */
+  unsigned short length;		/* number of samples per channel */
+  mad_fixed_t samples[2][1152];		/* PCM output samples [ch][sample] */
+};
+
 struct mad_synth {
   mad_fixed_t filter[2][2][2][16][8];	/* polyphase filterbank outputs */
   					/* [ch][eo][peo][s][v] */
 
   unsigned int phase;			/* current processing phase */
 
-  struct mad_pcm {
-    unsigned int samplerate;		/* sampling frequency (Hz) */
-    unsigned short channels;		/* number of channels */
-    unsigned short length;		/* number of samples per channel */
-    mad_fixed_t samples[2][1152];	/* PCM output samples */
-  } pcm;
+  struct mad_pcm pcm;			/* PCM output */
+};
+
+/* single channel PCM selector */
+enum {
+  MAD_PCM_CHANNEL_SINGLE = 0
+};
+
+/* dual channel PCM selector */
+enum {
+  MAD_PCM_CHANNEL_DUAL_1 = 0,
+  MAD_PCM_CHANNEL_DUAL_2 = 1
+};
+
+/* stereo PCM selector */
+enum {
+  MAD_PCM_CHANNEL_STEREO_LEFT  = 0,
+  MAD_PCM_CHANNEL_STEREO_RIGHT = 1
 };
 
 void mad_synth_init(struct mad_synth *);
@@ -765,7 +858,7 @@ void mad_synth_frame(struct mad_synth *, struct mad_frame const *);
 
 # endif
 
-/* Id: decoder.h,v 1.9 2001/04/05 04:57:11 rob Exp */
+/* Id: decoder.h,v 1.13 2001/11/03 03:57:11 rob Exp */
 
 # ifndef LIBMAD_DECODER_H
 # define LIBMAD_DECODER_H
@@ -776,10 +869,10 @@ enum mad_decoder_mode {
 };
 
 enum mad_flow {
-  MAD_FLOW_CONTINUE = 0x0000,
-  MAD_FLOW_STOP     = 0x0010,
-  MAD_FLOW_BREAK    = 0x0011,
-  MAD_FLOW_IGNORE   = 0x0020
+  MAD_FLOW_CONTINUE = 0x0000,	/* continue normally */
+  MAD_FLOW_STOP     = 0x0010,	/* stop decoding normally */
+  MAD_FLOW_BREAK    = 0x0011,	/* stop decoding and signal an error */
+  MAD_FLOW_IGNORE   = 0x0020	/* ignore the current frame */
 };
 
 struct mad_decoder {
@@ -803,7 +896,8 @@ struct mad_decoder {
 
   enum mad_flow (*input_func)(void *, struct mad_stream *);
   enum mad_flow (*header_func)(void *, struct mad_header const *);
-  enum mad_flow (*filter_func)(void *, struct mad_frame *);
+  enum mad_flow (*filter_func)(void *,
+			       struct mad_stream const *, struct mad_frame *);
   enum mad_flow (*output_func)(void *,
 			       struct mad_header const *, struct mad_pcm *);
   enum mad_flow (*error_func)(void *, struct mad_stream *, struct mad_frame *);
@@ -813,7 +907,9 @@ struct mad_decoder {
 void mad_decoder_init(struct mad_decoder *, void *,
 		      enum mad_flow (*)(void *, struct mad_stream *),
 		      enum mad_flow (*)(void *, struct mad_header const *),
-		      enum mad_flow (*)(void *, struct mad_frame *),
+		      enum mad_flow (*)(void *,
+					struct mad_stream const *,
+					struct mad_frame *),
 		      enum mad_flow (*)(void *,
 					struct mad_header const *,
 					struct mad_pcm *),
@@ -823,10 +919,14 @@ void mad_decoder_init(struct mad_decoder *, void *,
 		      enum mad_flow (*)(void *, void *, unsigned int *));
 int mad_decoder_finish(struct mad_decoder *);
 
-# define mad_decoder_options(decoder, opts)  ((decoder)->options = (opts))
+# define mad_decoder_options(decoder, opts)  \
+    ((void) ((decoder)->options = (opts)))
 
 int mad_decoder_run(struct mad_decoder *, enum mad_decoder_mode);
 int mad_decoder_message(struct mad_decoder *, void *, unsigned int *);
 
 # endif
 
+# ifdef __cplusplus
+}
+# endif
